<?php

class WPTestIncludesMeta extends WPTestCase {
	function setUp() {
		parent::setUp();
		$this->author = get_userdatabylogin(WP_USER_NAME);
		$this->post_ids = array();
		$this->meta_id = add_metadata( 'user', $this->author->ID, 'meta_key', 'meta_value' );
		$this->delete_meta_id = add_metadata( 'user', $this->author->ID, 'delete_meta_key', 'delete_meta_value' );
	}

	function tearDown() {
		parent::tearDown();
		foreach ($this->post_ids as $id)
			wp_delete_post($id);

		delete_metadata( 'user', $this->author->ID, 'meta_key', '', true );
		delete_metadata( 'user', $this->author->ID, 'delete_meta_key', '', true );
	}

	function _meta_sanitize_cb ( $meta_value, $meta_key, $meta_type ) {
		return 'sanitized';
	}

	function test_sanitize_meta() {
		if ( ! function_exists( 'register_meta' ) )
			return;

		$meta = sanitize_meta( 'some_meta', 'unsanitized', 'post' );
		$this->assertEquals( 'unsanitized', $meta );

		register_meta( 'post', 'some_meta', array( &$this, '_meta_sanitize_cb' ) );
		$meta = sanitize_meta( 'some_meta', 'unsanitized', 'post' );
		$this->assertEquals( 'sanitized', $meta );
	}

	function test_delete_metadata_by_mid() {
		if ( ! function_exists( 'delete_metadata_by_mid' ) || ! function_exists( 'get_metadata_by_mid' ) )
			return;

		// Let's try and delete a non-existing ID, non existing meta
		$this->assertFalse( delete_metadata_by_mid( 'user', 0 ) );
		$this->assertFalse( delete_metadata_by_mid( 'non_existing_meta', $this->delete_meta_id ) );

		// Now let's delete the real meta data
		$this->assertTrue( delete_metadata_by_mid( 'user', $this->delete_meta_id ) );

		// And make sure it's been deleted
		$this->assertFalse( get_metadata_by_mid( 'user', $this->delete_meta_id ) );

		// Make sure the caches are cleared
		$this->assertFalse( (bool) get_user_meta( $this->author->ID, 'delete_meta_key' ) );
	}

	function test_update_metadata_by_mid() {
		if ( ! function_exists( 'update_metadata_by_mid' ) || ! function_exists( 'get_metadata_by_mid') )
			return;

		// Setup
		$meta = get_metadata_by_mid( 'user', $this->meta_id );

		// Update the meta value
		$this->assertTrue( update_metadata_by_mid( 'user', $this->meta_id, 'meta_new_value' ) );
		$meta = get_metadata_by_mid( 'user', $this->meta_id );
		$this->assertEquals( 'meta_new_value', $meta->meta_value );

		// Update the meta value
		$this->assertTrue( update_metadata_by_mid( 'user', $this->meta_id, 'meta_new_value', 'meta_new_key' ) );
		$meta = get_metadata_by_mid( 'user', $this->meta_id );
		$this->assertEquals( 'meta_new_key', $meta->meta_key );

		// Update the key and value
		$this->assertTrue( update_metadata_by_mid( 'user', $this->meta_id, 'meta_value', 'meta_key' ) );
		$meta = get_metadata_by_mid( 'user', $this->meta_id );
		$this->assertEquals( 'meta_key', $meta->meta_key );
		$this->assertEquals( 'meta_value', $meta->meta_value );

		// Update the value that has to be serialized
		$this->assertTrue( update_metadata_by_mid( 'user', $this->meta_id, array( 'first', 'second' ) ) );
		$meta = get_metadata_by_mid( 'user', $this->meta_id );
		$this->assertEquals( array( 'first', 'second' ), $meta->meta_value );

		// Let's try some invalid meta data
		$this->assertFalse( update_metadata_by_mid( 'user', 0, 'meta_value' ) );
		$this->assertFalse( update_metadata_by_mid( 'user', $this->meta_id, 'meta_value', array('invalid', 'key' ) ) );

		// Let's see if caches get cleared after updates.
		$meta = get_metadata_by_mid( 'user', $this->meta_id );
		$first = get_user_meta( $meta->user_id, $meta->meta_key );
		$this->assertTrue( update_metadata_by_mid( 'user', $this->meta_id, 'other_meta_value' ) );
		$second = get_user_meta( $meta->user_id, $meta->meta_key );
		$this->assertFalse( $first === $second );
	}

	function test_metadata_exists() {
		if ( ! function_exists( 'metadata_exists' ) )
			return;

		$this->assertFalse( metadata_exists( 'user',  $this->author->ID, 'foobarbaz' ) );
		$this->assertTrue( metadata_exists( 'user',  $this->author->ID, 'meta_key' ) );
		$this->assertFalse( metadata_exists( 'user',  1234567890, 'foobarbaz' ) );
		$this->assertFalse( metadata_exists( 'user',  1234567890, 'meta_key' ) );
	}
}
